'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = parseContent;

var _falafel = require('falafel');

var _falafel2 = _interopRequireDefault(_falafel);

var _atomdoc = require('atomdoc');

var _atomdoc2 = _interopRequireDefault(_atomdoc);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _comments = [];

function _getNodeType(node) {
  if (node.type !== 'Identifier') return node.type;
  return node.parent.type;
}

function _nodeTest(node) {
  var nodeType = _getNodeType(node);
  var functionTypes = ['FunctionDeclaration', 'ArrowFunctionExpression', 'VariableDeclaration', 'MethodDefinition'];
  return functionTypes.includes(nodeType);
}

function _getFunctionName(node) {
  var nodeType = _getNodeType(node);
  var _node = nodeType !== node.type ? node.parent : node;
  switch (nodeType) {
    case 'MethodDefinition':
      return _node.key.name;
    case 'FunctionDeclaration':
      return _node.id.name;
    case 'VariableDeclaration':
      return _node.declarations[0].id.name;
    case 'ArrowFunctionExpression':
      if (_node.parent.id) return _node.parent.id.name;
    // eslint-disable-next-line no-fallthrough
    default:
      return null;
  }
}

/**
 *  Private: adjusts contents of block comments so the base indent is 0.
 *
 *  * `str` {String} block comment.
 *
 *  Returns {String} adjusted content.
 */
function _fixIndent(str) {
  var indentRegEx = /^(\s)*/;
  var indent = str.match(indentRegEx)[0];
  return str.replace(new RegExp(indent, ['g']), '\n').trim();
}

function _removeBlockAstrisks(str) {
  if (str.match(/^\*/)) {
    return str.replace(/^\s*\*/gm, '');
  }
  return str;
}

function _isAtomDocComment(block, text) {
  if (!block) return false;
  return Boolean(text.match(/^([\*\s]*|Private:|Public:|Internal:)*/));
}

function _parseComment(block, text, start, end) {
  if (_isAtomDocComment(block, text)) {
    var value = _fixIndent(_removeBlockAstrisks(text));
    var atomDoc = _atomdoc2.default.parse(value);
    _comments.push(Object.assign({}, atomDoc, { start: start, end: end }));
  }
}

function _getNextCommentIndex(comments, endPos) {
  return comments.findIndex(function (comment) {
    return comment.start <= endPos;
  });
}

function parseContent(content) {
  return new Promise(function (resolve) {
    var commentIndex = -1;
    var captureNextFunctionName = false;
    _comments.length = 0;
    (0, _falafel2.default)(content, {
      sourceType: 'module',
      ecmaVersion: '6',
      onComment: _parseComment
    }, function (node) {
      if (captureNextFunctionName && _nodeTest(node)) {
        _comments[commentIndex].name = _getFunctionName(node);
        captureNextFunctionName = false;
      }
      var nextCommentIndex = _getNextCommentIndex(_comments, node.end);
      if (nextCommentIndex > commentIndex) {
        commentIndex = nextCommentIndex;
        if (_nodeTest(node)) {
          _comments[commentIndex].name = _getFunctionName(node);
        } else {
          captureNextFunctionName = true;
        }
      }
      if (node.end === content.length) resolve(_comments);
    });
  });
}
module.exports = exports['default'];